@@ -318,7 +318,7 @@ nw_crypt(const char *txt, const char *salt)
     return des_fcrypt(txt, salt, w32_crypt_buffer);
 #else
     Perl_croak(aTHX_ "The crypt() function is not implemented on NetWare\n");
-    return NULL;
+    return Nullch;
 #endif
 }
 
@@ -567,7 +567,7 @@ nw_rmdir(const char *dir)
 }
 
 DIR *
-nw_opendir(char *filename)
+nw_opendir(const char *filename)
 {
 	char	*buff = NULL;
 	int		len = 0;
@@ -1001,7 +1001,7 @@ do_spawn2(char *cmd, int exectype)
 	    if (*s)
 		*s++ = '\0';
 	}
-	*a = NULL;
+	*a = Nullch;
 	if (argv[0]) {
 	    switch (exectype) {
 			case EXECF_SPAWN:
@@ -1031,7 +1031,7 @@ do_spawn2(char *cmd, int exectype)
 	while (++i < nw_perlshell_items)
 	    argv[i] = nw_perlshell_vec[i];
 	argv[i++] = cmd;
-	argv[i] = NULL;
+	argv[i] = Nullch;
 	switch (exectype) {
 		case EXECF_SPAWN:
 			status = nw_spawnvp(P_WAIT, argv[0], (char **)argv);
