@@ -8,6 +8,7 @@ BEGIN {
   require 'regen_lib.pl';
 }
 
+use strict;
 
 #
 # See database of global and static function prototypes in embed.fnc
@@ -144,6 +145,27 @@ removed without notice.\n\n" if $flags =~ /x/;
     print $fh "=for hackers\nFound in file $file\n\n";
 }
 
+sub readonly_header (*) {
+    my $fh = shift;
+    print $fh <<"_EOH_";
+-*- buffer-read-only: t -*-
+
+!!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
+This file is built by $0 extracting documentation from the C source
+files.
+
+_EOH_
+}
+
+sub readonly_footer (*) {
+    my $fh = shift;
+    print $fh <<'_EOF_';
+=cut
+
+ ex: set ro:
+_EOF_
+}
+
 my $file;
 # glob() picks up docs from extra .c or .h files that may be in unclean
 # development trees.
@@ -197,6 +219,8 @@ for (sort keys %docfuncs) {
     warn "Unable to place $_!\n";
 }
 
+readonly_header(\*DOC);
+
 print DOC <<'_EOB_';
 =head1 NAME
 
@@ -254,6 +278,7 @@ perlguts(1), perlxs(1), perlxstut(1), perlintern(1)
 
 _EOE_
 
+readonly_footer(\*DOC);
 
 close(DOC) or die "Error closing pod/perlapi.pod: $!";
 
@@ -261,6 +286,7 @@ safer_unlink "pod/perlintern.pod";
 open(GUTS, ">pod/perlintern.pod") or
 		die "Unable to create pod/perlintern.pod: $!\n";
 binmode GUTS;
+readonly_header(\*GUTS);
 print GUTS <<'END';
 =head1 NAME
 
@@ -299,5 +325,6 @@ document their functions.
 perlguts(1), perlapi(1)
 
 END
+readonly_footer(\*GUTS);
 
 close GUTS or die "Error closing pod/perlintern.pod: $!";
