@@ -33,8 +33,8 @@ apirevision=''
 apisubversion=''
 apiversion=''
 ar='arm-epoc-pe-ar'
-archlib='/usr/lib/perl/5.9.4/epoc'
-archlibexp='/usr/lib/perl/5.9.4/epoc'
+archlib='/usr/lib/perl/5.9.5/epoc'
+archlibexp='/usr/lib/perl/5.9.5/epoc'
 archname64=''
 archname='epoc'
 archobjs='epoc.o epocish.o epoc_stubs.o'
@@ -140,6 +140,7 @@ d_cuserid='undef'
 d_dbl_dig='undef'
 d_dbminitproto='undef'
 d_difftime='define'
+d_dir_dd_fd='undef'
 d_dirfd='undef'
 d_dirnamlen='undef'
 d_dlerror='undef'
@@ -270,6 +271,7 @@ d_ldbl_dig='undef'
 d_link='undef'
 d_llseek='undef'
 d_localtime_r='undef'
+d_localtime_r_needs_tzset='undef'
 d_locconv='undef'
 d_lockf='undef'
 d_longdbl='undef'
@@ -738,8 +740,8 @@ pmake=''
 pr=''
 prefix=''
 prefixexp=''
-privlib='/usr/lib/perl/5.9.4'
-privlibexp='/usr/lib/perl/5.9.4'
+privlib='/usr/lib/perl/5.9.5'
+privlibexp='/usr/lib/perl/5.9.5'
 procselfexe=''
 prototype='define'
 ptrsize='4'
@@ -794,11 +796,11 @@ sig_num='0'
 sig_num_init='0, 0'
 sig_size='1'
 signal_t='void'
-sitearch='/usr/lib/perl/site_perl/5.9.4/epoc'
-sitearchexp='/usr/lib/perl/site_perl/5.9.4/epoc'
-sitelib='/usr/lib/perl/site_perl/5.9.4/'
+sitearch='/usr/lib/perl/site_perl/5.9.5/epoc'
+sitearchexp='/usr/lib/perl/site_perl/5.9.5/epoc'
+sitelib='/usr/lib/perl/site_perl/5.9.5/'
 sitelib_stem='/usr/lib/perl/site_perl'
-sitelibexp='/usr/lib/perl/site_perl/5.9.4/'
+sitelibexp='/usr/lib/perl/site_perl/5.9.5/'
 siteprefix=''
 siteprefixexp=''
 sizesize='4'
@@ -879,7 +881,7 @@ vendorlib_stem=''
 vendorlibexp=''
 vendorprefix=''
 vendorprefixexp=''
-version='5.9.4'
+version='5.9.5'
 versiononly='undef'
 vi=''
 voidflags='15'
@@ -1011,7 +1013,7 @@ vendorlib_stem=''
 vendorlibexp=''
 vendorprefix=''
 vendorprefixexp=''
-version='5.9.4'
+version='5.9.5'
 vi=''
 voidflags='15'
 xlibpth=''
@@ -1153,3 +1155,19 @@ nvfformat='"f"'
 nvgformat='"g"'
 sSCNfldbl=''
 uquadtype='uint64_t'
+d_builtin_choose_expr='undef'
+d_builtin_expect='undef'
+d_ctermid='undef'
+d_inc_version_list='undef'
+d_libm_lib_version='0'
+d_printf_format_null='undef'
+d_pseudofork='undef'
+d_signbit='undef'
+d_sitearch='undef'
+db_version_major='0'
+db_version_minor='0'
+db_version_patch='0'
+targetarch=''
+vendorarch=''
+vendorarchexp=''
+
