@@ -3,7 +3,7 @@ use strict;
 use warnings;
 no warnings 'redefine';
 
-our $VERSION = do { my @r = ( q$Revision: 2.4 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
+our $VERSION = do { my @r = ( q$Revision: 2.5 $ =~ /\d+/g ); sprintf "%d." . "%02d" x $#r, @r };
 use Encode qw(find_encoding encode_utf8 decode_utf8);
 use MIME::Base64;
 use Carp;
@@ -174,12 +174,13 @@ sub _encode_b {
 
 sub _encode_q {
     my $chunk = shift;
+    $chunk = encode_utf8($chunk);
     $chunk =~ s{
         ([^0-9A-Za-z])
            }{
            join("" => map {sprintf "=%02X", $_} unpack("C*", $1))
            }egox;
-    return decode_utf8( HEAD . 'Q?' . $chunk . TAIL );
+    return HEAD . 'Q?' . $chunk . TAIL;
 }
 
 1;
