@@ -1,7 +1,7 @@
 package PerlIO::encoding;
 
 use strict;
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 our $DEBUG = 0;
 $DEBUG and warn __PACKAGE__, " called by ", join(", ", caller), "\n";
 
@@ -25,6 +25,8 @@ PerlIO::encoding - encoding layer
 
 =head1 SYNOPSIS
 
+  use PerlIO::encoding;
+
   open($f, "<:encoding(foo)", "infoo");
   open($f, ">:encoding(bar)", "outbar");
 
@@ -33,16 +35,16 @@ PerlIO::encoding - encoding layer
 
 =head1 DESCRIPTION
 
-Open a filehandle with a transparent encoding filter.
+This PerlIO layer opens a filehandle with a transparent encoding filter.
 
-On input, convert the bytes expected to be in the specified
+On input, it converts the bytes expected to be in the specified
 character set and encoding to Perl string data (Unicode and
-Perl's internal Unicode encoding, UTF-8).  On output, convert
+Perl's internal Unicode encoding, UTF-8).  On output, it converts
 Perl string data into the specified character set and encoding.
 
-When the layer is pushed the current value of C<$PerlIO::encoding::fallback>
-is saved and used as the CHECK argument when calling the Encode methods encode()
-and decode().
+When the layer is pushed, the current value of C<$PerlIO::encoding::fallback>
+is saved and used as the CHECK argument when calling the Encode methods
+encode() and decode().
 
 =head1 SEE ALSO
 
