@@ -8,7 +8,7 @@ BEGIN {
     require DynaLoader;
 
     @ISA = qw|Exporter DynaLoader|;
-    $VERSION = '0.38';
+    $VERSION = '0.30';
     $XS_VERSION = $VERSION;
     $VERSION = eval $VERSION;
 
@@ -310,10 +310,9 @@ a name that can be passed to system calls and external programs.
 
 =item Win32::DomainName()
 
-[CORE] Returns the name of the Microsoft Network domain or workgroup
-that the owner of the current perl process is logged into.  The
-"Workstation" service must be running to determine this
-information.  This function does B<not> work on Windows 9x.
+[CORE] Returns the name of the Microsoft Network domain that the
+owner of the current perl process is logged into.  This function does
+B<not> work on Windows 9x.
 
 =item Win32::ExpandEnvironmentStrings(STRING)
 
@@ -403,25 +402,15 @@ If supported by the core Perl version, this function will return an
 ANSI path name for the current directory if the long pathname cannot
 be represented in the system codepage.
 
-=item Win32::GetCurrentProcessId()
-
-Returns the process identifier of the current process.  Until the
-process terminates, the process identifier uniquely identifies the
-process throughout the system.
-
-The current process identifier is normally also available via the
-predefined $$ variable.  Under fork() emulation however $$ may contain
-a pseudo-process identifier that is only meaningful to the Perl
-kill(), wait() and waitpid() functions.  The
-Win32::GetCurrentProcessId() function will always return the regular
-Windows process id, even when called from inside a pseudo-process.
-
 =item Win32::GetCurrentThreadId()
 
 Returns the thread identifier of the calling thread.  Until the thread
 terminates, the thread identifier uniquely identifies the thread
 throughout the system.
 
+Note: the current process identifier is available via the predefined
+$$ variable.
+
 =item Win32::GetFileVersion(FILENAME)
 
 Returns the file version number from the VERSIONINFO resource of
@@ -536,7 +525,7 @@ elements are, respectively: An arbitrary descriptive string, the major
 version number of the operating system, the minor version number, the
 build number, and a digit indicating the actual operating system.
 For the ID, the values are 0 for Win32s, 1 for Windows 9X/Me and 2 for
-Windows NT/2000/XP/2003/Vista.  In scalar context it returns just the ID.
+Windows NT/2000/XP/2003.  In scalar context it returns just the ID.
 
 Currently known values for ID MAJOR and MINOR are as follows:
 
@@ -582,7 +571,7 @@ constants.
 PRODUCTTYPE provides additional information about the system.  It should
 be one of the following integer values:
 
-    1 - Workstation (NT 4, 2000 Pro, XP Home, XP Pro, Vista)
+    1 - Workstation (NT 4, 2000 Pro, XP Home, XP Pro)
     2 - Domaincontroller
     3 - Server
 
@@ -597,7 +586,7 @@ GetOSVersion() in list context.
 
 Currently the possible values for the OS name are
 
- Win32s Win95 Win98 WinMe WinNT3.51 WinNT4 Win2000 WinXP/.Net Win2003 WinVista
+ Win32s Win95 Win98 WinMe WinNT3.51 WinNT4 Win2000 WinXP/.Net Win2003
 
 This routine is just a simple interface into GetOSVersion().  More
 specific or demanding situations should use that instead.  Another
