@@ -116,17 +116,17 @@ case "$osvers" in
 
 *)
         objformat=`/usr/bin/objformat`
-        if [ x$objformat = xelf ]; then
-            libpth="/usr/lib /usr/local/lib"
-            glibpth="/usr/lib /usr/local/lib"
-            ldflags="-Wl,-E "
-            lddlflags="-shared "
-        else
+        if [ x$objformat = xaout ]; then
             if [ -e /usr/lib/aout ]; then
                 libpth="/usr/lib/aout /usr/local/lib /usr/lib"
                 glibpth="/usr/lib/aout /usr/local/lib /usr/lib"
             fi
             lddlflags='-Bshareable'
+        else
+            libpth="/usr/lib /usr/local/lib"
+            glibpth="/usr/lib /usr/local/lib"
+            ldflags="-Wl,-E "
+            lddlflags="-shared "
         fi
         cccdlflags='-DPIC -fPIC'
         ;;
