@@ -1,10 +1,11 @@
 package ExtUtils::MM_UWIN;
 
 use strict;
-our $VERSION = 6.48;
+use vars qw($VERSION @ISA);
+$VERSION = 0.04;
 
 require ExtUtils::MM_Unix;
-our @ISA = qw(ExtUtils::MM_Unix);
+@ISA = qw(ExtUtils::MM_Unix);
 
 
 =head1 NAME
