@@ -3,12 +3,14 @@ package ExtUtils::MY;
 use strict;
 require ExtUtils::MM;
 
-our $VERSION = 6.48;
-our @ISA = qw(ExtUtils::MM);
+use vars qw(@ISA $VERSION);
+$VERSION = 0.03;
+@ISA = qw(ExtUtils::MM);
 
 {
     package MY;
-    our @ISA = qw(ExtUtils::MY);
+    use vars qw(@ISA);
+    @ISA = qw(ExtUtils::MY);
 }
 
 sub DESTROY {}
