@@ -27,7 +27,7 @@ Please see the L<Module::Build> for the general docs.
 
 =head1 AUTHOR
 
-Ken Williams <ken@cpan.org>
+Ken Williams <kwilliams@cpan.org>
 
 =head1 SEE ALSO
 
