@@ -1,12 +1,13 @@
 package Text::Wrap;
 
+use warnings::register;
 require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(wrap fill);
 @EXPORT_OK = qw($columns $break $huge);
 
-$VERSION = 2006.0711;
+$VERSION = 2006.1117;
 
 use vars qw($VERSION $columns $debug $break $huge $unexpand $tabstop
 	$separator $separator2);
@@ -62,7 +63,7 @@ sub wrap
 		} elsif ($huge eq 'die') {
 			die "couldn't wrap '$t'";
 		} elsif ($columns < 2) {
-			warn "Increasing \$Text::Wrap::columns from $columns to 2";
+			warnings::warnif "Increasing \$Text::Wrap::columns from $columns to 2";
 			$columns = 2;
 			return ($ip, $xp, @t);
 		} else {
