@@ -168,6 +168,10 @@ PERL_EXPORT_C void PerlIO_list_free(pTHX_ PerlIO_list_t *list);
 PERL_EXPORT_C void PerlIO_list_push(pTHX_ PerlIO_list_t *list, PerlIO_funcs *funcs, SV *arg);
 PERL_EXPORT_C void PerlIO_list_free(pTHX_ PerlIO_list_t *list);
 
+/* PerlIO_teardown doesn't need exporting, but the EXTERN_C is needed
+ * for compiling as C++.  Must also match with what perl.h says. */
+EXTERN_C void PerlIO_teardown(pTHX);
+
 /*--------------------------------------------------------------------------------------*/
 /* Generic, or stub layer functions */
 
