@@ -20,6 +20,8 @@ castflags='0'
 cc='gcc'
 cccdlflags=''
 ccdlflags=''
+cf_by='root@localhost'
+cf_time='Thu Jan  1 00:00:00 GMT 1970'
 charsize='1'
 clocktype='clock_t'
 cpp_stuff='42'
@@ -60,6 +62,8 @@ d_attribute_noreturn='undef'
 d_attribute_pure='undef'
 d_attribute_unused='undef'
 d_attribute_warn_unused_result='undef'
+d_builtin_expect='undef'
+d_builtin_choose_expr='undef'
 d_bcmp='undef'
 d_bcopy='undef'
 d_bsd='undef'
@@ -82,6 +86,7 @@ d_cplusplus='undef'
 d_crypt='undef'
 d_crypt_r='undef'
 d_csh='undef'
+d_ctermid='undef'
 d_ctermid_r='undef'
 d_ctime_r='undef'
 d_c99_variadic_macros='undef'
@@ -89,6 +94,7 @@ d_cuserid='undef'
 d_dbl_dig='undef'
 d_dbminitproto='undef'
 d_difftime='undef'
+d_dir_dd_fd='undef'
 d_dirfd='undef'
 d_dirnamlen='define'
 d_dlerror='undef'
@@ -125,7 +131,7 @@ d_fds_bits='undef'
 d_fgetpos='undef'
 d_finite='undef'
 d_finitel='undef'
-d_flexfnam='undef'
+d_flexfnam='define'
 d_flock='undef'
 d_flockproto='undef'
 d_fork='undef'
@@ -283,7 +289,9 @@ d_phostname='undef'
 d_pipe='undef'
 d_poll='undef'
 d_portable='undef'
+d_printf_format_null='undef'
 d_procselfexe='undef'
+d_pseudofork='undef'
 d_pthread_atfork='undef'
 d_pthread_attr_setscope='undef'
 d_pthread_yield='undef'
@@ -363,6 +371,7 @@ d_shmctl='undef'
 d_shmdt='undef'
 d_shmget='undef'
 d_sigaction='undef'
+d_signbit='undef'
 d_sigprocmask='undef'
 d_sigsetjmp='undef'
 d_sitearch='define'
@@ -728,6 +737,7 @@ so='o'
 socksizetype='unsigned int'
 srand48_r_proto='0'
 srandom_r_proto='0'
+src=''
 ssizetype=int
 startperl=''
 stdchar=char
@@ -759,7 +769,7 @@ uquadtype='uint64_t'
 use5005threads='undef'
 use64bitall='undef'
 use64bitint='undef'
-usecrosscompile='define'
+usecrosscompile='undef'
 usedl='undef'
 usefaststdio='undef'
 useithreads='undef'
