@@ -7,6 +7,7 @@ require Exporter;
 use strict;
 use File::Path;
 use File::Basename;
+use MakeMaker::Test::Utils;
 
 my %Files = (
              'Recurs/Makefile.PL'          => <<'END',
@@ -25,10 +26,21 @@ WriteMakefile(
     NAME => 'Recurs::prj2',
     VERSION => 1.00,
 );
+END
+
+             # Check if a test failure in a subdir causes make test to fail
+             'Recurs/prj2/t/fail.t'         => <<'END',
+#!/usr/bin/perl -w
+
+print "1..1\n";
+print "not ok 1\n";
 END
             );
 
 sub setup_recurs {
+    setup_mm_test_root();
+    chdir 'MM_TEST_ROOT:[t]' if $^O eq 'VMS';
+
     while(my($file, $text) = each %Files) {
         # Convert to a relative, native file path.
         $file = File::Spec->catfile(File::Spec->curdir, split m{\/}, $file);
