@@ -23,7 +23,7 @@ sub casetest {
     my $file = File::Spec->catfile(File::Spec->catdir(File::Spec->updir,
 						      "lib", "unicore", "To"),
 				   "$base.pl");
-    my $simple = do $file;
+    my $simple = do $file or die $@;
     my %simple;
     for my $i (split(/\n/, $simple)) {
 	my ($k, $v) = split(' ', $i);
