@@ -4,15 +4,23 @@ _o='.o'
 afs='false'
 afsroot='/afs'
 alignbytes='4'
+aphostname='/bin/hostname'
 archlib='/usr/local/lib/perl5/5.9/unknown'
 archlibexp='/usr/local/lib/perl5/5.9/unknown'
 archname='unknown'
 asctime_r_proto='0'
 bin='/usr/local/bin'
+binexp='/usr/local/bin'
 byteorder='1234'
 castflags='0'
+cf_by='root@localhost'
+cf_time='Thu Jan  1 00:00:00 GMT 1970'
 charsize='1'
 clocktype='clock_t'
+cpplast='-'
+cppminus='-'
+cpprun='cc -E'
+cppstdin='cc -E'
 cpp_stuff='42'
 crypt_r_proto='0'
 ctermid_r_proto='0'
@@ -71,6 +79,7 @@ d_cplusplus='undef'
 d_crypt='undef'
 d_crypt_r='undef'
 d_csh='undef'
+d_ctermid='undef'
 d_ctermid_r='undef'
 d_ctime_r='undef'
 d_c99_variadic_macros='undef'
@@ -78,6 +87,7 @@ d_cuserid='undef'
 d_dbl_dig='undef'
 d_dbminitproto='undef'
 d_difftime='undef'
+d_dir_dd_fd='undef'
 d_dirfd='undef'
 d_dirnamlen='undef'
 d_dlerror='undef'
@@ -212,6 +222,7 @@ d_ldbl_dig='undef'
 d_libm_lib_version='undef'
 d_link='undef'
 d_localtime_r='undef'
+d_localtime_r_needs_tzset='undef'
 d_locconv='undef'
 d_lockf='undef'
 d_longdbl='undef'
@@ -271,7 +282,9 @@ d_phostname='undef'
 d_pipe='undef'
 d_poll='undef'
 d_portable='undef'
+d_printf_format_null='undef'
 d_procselfexe='undef'
+d_pseudofork='undef'
 d_pthread_atfork='undef'
 d_pthread_attr_setscope='undef'
 d_pthread_yield='undef'
@@ -351,6 +364,7 @@ d_shmctl='undef'
 d_shmdt='undef'
 d_shmget='undef'
 d_sigaction='undef'
+d_signbit='undef'
 d_sigprocmask='undef'
 d_sigsetjmp='undef'
 d_sitearch='undef'
@@ -444,6 +458,9 @@ d_writev='undef'
 d_xenix='undef'
 db_hashtype='u_int32_t'
 db_prefixtype='size_t'
+db_version_major='0'
+db_version_minor='0'
+db_version_patch='0'
 defvoidused=1
 direntrytype='struct dirent'
 doublesize='8'
@@ -462,7 +479,9 @@ fflushall='undef'
 firstmakefile='makefile'
 fpossize='4'
 fpostype=int
-freetype=int
+freetype=void
+full_csh=''
+full_sed=''
 getgrent_r_proto='0'
 getgrgid_r_proto='0'
 getgrnam_r_proto='0'
@@ -599,6 +618,7 @@ lseektype=int
 mad='undef'
 malloctype='int*'
 malloctype='void *'
+mmaptype='void *'
 modetype='mode_t'
 modetype=int
 multiarch='undef'
@@ -617,9 +637,13 @@ nvsize='8'
 nvtype='double'
 o_nonblock='O_NONBLOCK'
 obj_ext='.o'
+old_pthread_create_joinable=''
 optimize='-O2'
 orderlib='false'
 osname='unknown'
+osvers='unknown'
+otherlibdirs=' '
+package='perl5'
 phostname='hostname'
 pidtype=int
 privlib='/usr/local/lib/perl5/5.9'
@@ -662,18 +686,26 @@ setnetent_r_proto='0'
 setprotoent_r_proto='0'
 setpwent_r_proto='0'
 setservent_r_proto='0'
+sh='/bin/sh'
 shmattype='void *'
-shortsize=2
+shortsize='2'
 sig_name_init='0'
 sig_num_init='0'
 sig_size='1'
 signal_t=int
+sitearch='/usr/local/lib/perl5/5.9/unknown'
+sitearchexp='/usr/local/lib/perl5/5.9/unknown'
+sitelib='/usr/local/lib/perl5/5.9'
+sitelib_stem='/usr/local/lib/perl5'
+sitelibexp='/usr/local/lib/perl5/5.9'
 sizesize=4
 sizetype='size_t'
 socksizetype='int'
 srand48_r_proto='0'
 srandom_r_proto='0'
+src='.'
 ssizetype=int
+startperl='#!perl'
 stdchar=char
 stdio_base='((fp)->_IO_read_base)'
 stdio_bufsiz='((fp)->_IO_read_end - (fp)->_IO_read_base)'
@@ -682,6 +714,7 @@ stdio_filbuf=''
 stdio_ptr='((fp)->_IO_read_ptr)'
 stdio_stream_array=''
 strerror_r_proto='0'
+targetarch=''
 timetype=time_t
 tmpnam_r_proto='0'
 touch='touch'
@@ -731,5 +764,9 @@ uvsize='4'
 uvtype='unsigned long'
 uvuformat='"lu"'
 uvxformat='"lx"'
+vendorarch=''
+vendorarchexp=''
+vendorlib_stem=''
+vendorlibexp=''
 versiononly='undef'
 voidflags=1
